local addonName, addonTable = ...; -- Pulls back the Addon-Local Variables and store them locally.
local FORCE_DEBUG = false;

if LunaEclipse_SpecManager:isCompatableWOWVersion() then
    local moduleName = "EquipManager";
    local EquipManager = LunaEclipse_SpecManager:NewModule(moduleName, "AceConsole-3.0", "AceEvent-3.0");
    addonTable.EquipManager = EquipManager;

    local equipSuccess = true;

	local function stripIconPath(icon)
		local returnValue = icon;

		if icon then
			local iconString = string.upper(icon);

			if string.find(iconString, "INTERFACE\\ICONS\\") then
				returnValue = string.sub(icon, string.len("INTERFACE\\ICONS\\") + 1, -1);
			end
		end

		return returnValue;	
	end

	local function EquipmentSet_Equipped(specInfo)
		local equipmentSetID = C_EquipmentSet.GetEquipmentSetID(specInfo.name);
		local setID, isEquipped = select(3, C_EquipmentSet.GetEquipmentSetInfo(equipmentSetID));
		
        return (setID and isEquipped) or false;	
    end

    local function EquipmentSet_Exists(specInfo)
		local equipmentSetID = C_EquipmentSet.GetEquipmentSetID(specInfo.name);
        local setID = select(3, C_EquipmentSet.GetEquipmentSetInfo(equipmentSetID));

        return (setID ~= nil) or false;
    end

    function EquipManager:ChatSpamFilter(event, msg, ...)
        if string.find(msg, string.gsub(ERR_LEARN_ABILITY_S:gsub("%.", "%."), "%%s", "(.*)"))
            or string.find(msg, string.gsub(ERR_LEARN_SPELL_S:gsub("%.", "%."), "%%s", "(.*)"))
            or string.find(msg, string.gsub(ERR_SPELL_UNLEARNED_S:gsub("%.", "%."), "%%s", "(.*)"))
            or string.find(msg, string.gsub(ERR_LEARN_PASSIVE_S:gsub("%.", "%."), "%%s", "(.*)")) then
                return true;
        else
            return false, msg, ...;
        end
    end

    function EquipManager:Load(event, specInfo)
        local profile = addonTable.addonSettings.profile;
		
        if specInfo and (profile and profile.AddonSettings.enableEquipManager) and (event ~= addonTable.LESM_LEFT_COMBAT or not equipSuccess) then
            if not EquipmentSet_Exists(specInfo) then
                EquipManager:Save(event, specInfo);
            elseif LunaEclipse_SpecManager:InCombat() then
				equipSuccess = false;
			elseif EquipmentSet_Exists(specInfo) and not EquipmentSet_Equipped(specInfo) then
				local equipmentSetID = C_EquipmentSet.GetEquipmentSetID(specInfo.name);

				-- Only attempt to use the Equipment Set if there is none assigned to be used for the specialization
				if not C_EquipmentSet.GetEquipmentSetForSpec(specInfo.index) then
					equipSuccess = C_EquipmentSet.UseEquipmentSet(equipmentSetID);
				end
            end
        end
    end

    function EquipManager:Save(event, specInfo)
        local profile = addonTable.addonSettings.profile;
		
        if specInfo and (profile and profile.AddonSettings.enableEquipManager) then
			local equipmentSetID = C_EquipmentSet.GetEquipmentSetID(specInfo.name);
			
			C_EquipmentSet.SaveEquipmentSet(equipmentSetID, LibIconPath_getIDByName(stripIconPath(specInfo.icon)));
			C_EquipmentSet.AssignSpecToEquipmentSet(equipmentSetID, specInfo.index);
        end
    end

    function EquipManager:OnEnable()
        self:RegisterMessage(addonTable.LESM_LOAD, "Save");
        self:RegisterMessage(addonTable.LESM_SPEC_ADDED, "Load");
        self:RegisterMessage(addonTable.LESM_SPEC_CHANGE_START, "Save");
        self:RegisterMessage(addonTable.LESM_SPEC_CHANGE_FINISH, "Load");
        self:RegisterMessage(addonTable.LESM_SPEC_REMOVED, "Save");
        self:RegisterMessage(addonTable.LESM_LEFT_COMBAT, "Load");

        ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", EquipManager.ChatSpamFilter);
    end

    function EquipManager:OnDisable()
        ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", EquipManager.ChatSpamFilter);

        self:UnregisterMessage(addonTable.LESM_LOAD);
        self:UnregisterMessage(addonTable.LESM_SPEC_ADDED);
        self:UnregisterMessage(addonTable.LESM_SPEC_CHANGE_START);
        self:UnregisterMessage(addonTable.LESM_SPEC_CHANGE_FINISH);
        self:UnregisterMessage(addonTable.LESM_SPEC_REMOVED);
        self:UnregisterMessage(addonTable.LESM_LEFT_COMBAT);
    end
end