local addonName, addonTable = ...; -- Pulls back the Addon-Local Variables and store them locally.
local FORCE_DEBUG = false;

if LunaEclipse_SpecManager:isCompatableWOWVersion() then
    local moduleName = "OverlayManager";
    local OverlayManager = LunaEclipse_SpecManager:NewModule(moduleName, "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0");
    addonTable.OverlayManager = OverlayManager;

    local REAGENTBANK_CONTAINER = -3;
    local KEYRING_CONTAINER = -2;
    local BANK_CONTAINER = -1;
    local BACKPACK_CONTAINER = 0;

    local INVENTORY_SLOTS = {
        [1] = "HeadSlot",
        [2] = "NeckSlot",
        [3] = "ShoulderSlot",
        [4] = "ShirtSlot",
        [5] = "ChestSlot",
        [6] = "WaistSlot",
        [7] = "LegsSlot",
        [8] = "FeetSlot",
        [9] = "WristSlot",
        [10] = "HandsSlot",
        [11] = "Finger0Slot",
        [12] = "Finger1Slot",
        [13] = "Trinket0Slot",
        [14] = "Trinket1Slot",
        [15] = "BackSlot",
        [16] = "MainHandSlot",
        [17] = "SecondaryHandSlot",
        [18] = nil;
        [19] = "TabardSlot"
    };

    local textureInBank = { red = 0, green = 1, blue = 1, alpha = 0.4 };
    local textureInBags = { red = 0, green = 1, blue = 0, alpha = 0.4 };
    local textureNotUsed = { red = 1, green = 1, blue = 0, alpha = 0.4 };
    local textureNotAvailable = { red = 1, green = 0, blue = 0, alpha = 0.4 };

    local bankContents = {};
    
    local function getEquipmentSlotFrame(ID)
        local inventorySlot = INVENTORY_SLOTS[tonumber(ID)];

        return (inventorySlot ~= nil and _G[string.join("", "Character", inventorySlot)]) or nil;
    end

    local function Bank_ContainerUpdate(bagID)
        if bagID and type(bagID) == "number" then
            local counter = GetContainerNumSlots(bagID);
 
            while counter > 0 do         
                local itemID = GetContainerItemID(bagID, counter);

                if itemID then
                    bankContents[itemID] = true;
                end
            
                counter = counter - 1;
            end
        end
    end

    local function Texture_Tint(targetFrame, colorTable)
        if targetFrame.LESM_Overlay then 
            if colorTable then
                targetFrame.LESM_Overlay:SetTexture(colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha);
            else
                targetFrame.LESM_Overlay:SetTexture(0, 0, 0, 0);
            end
        end
    end

    local function Texture_Hide(targetFrame)
        if targetFrame.LESM_Overlay then
            Texture_Tint(targetFrame);
        end
    end

    local function Texture_ChoseTint(targetFrame, itemLocation, currentItemID, itemID)
        if targetFrame and itemLocation then
            local player, bank, bags, slot, bag = EquipmentManager_UnpackLocation(itemLocation);

            if player and not bags then
                Texture_Hide(targetFrame);
            elseif bags and not bank then
                Texture_Tint(targetFrame, textureInBags);
            elseif bank or bankContents[itemID] then
                Texture_Tint(targetFrame, textureInBank);
            else
                Texture_Tint(targetFrame, textureNotAvailable);
            end
        elseif targetFrame and not itemLocation then
            if currentItemID == itemID then 
                Texture_Hide(targetFrame);
            else	
                Texture_Tint(targetFrame, textureNotUsed);
            end 
        end
    end

    local function Texture_Setup(targetFrame)
        if targetFrame and not targetFrame.LESM_Overlay then
            targetFrame.LESM_Overlay = targetFrame:CreateTexture(nil, "OVERLAY");
            targetFrame.LESM_Overlay:SetAllPoints(targetFrame);
        end
   
        Texture_Hide(targetFrame);
    end

    function OverlayManager:Overlay_Hide()
        for ID, inventorySlot in pairs(INVENTORY_SLOTS) do
            Texture_Hide(getEquipmentSlotFrame(ID));
        end
    end

    function OverlayManager:Overlay_Show()
        local profile = addonTable.addonSettings.profile;
		
        if PaperDollEquipmentManagerPane:IsShown() and (profile and profile.AddonSettings.useOverlay) then        
            local selectedSetName = PaperDollEquipmentManagerPane.selectedSetName;

            if selectedSetName then	        
                local setItems = GetEquipmentSetItemIDs(selectedSetName);
                local setLocations = GetEquipmentSetLocations(selectedSetName);
                
                for ID, inventorySlot in pairs(INVENTORY_SLOTS) do
                    local itemID = setItems[tonumber(ID)] or 0;
                    local currentItemID = GetInventoryItemID("player", GetInventorySlotInfo(inventorySlot)) or 0;
                    local itemLocation = setLocations[tonumber(ID)];
                    local targetFrame = getEquipmentSlotFrame(ID);                    	
                    
                    Texture_Setup(targetFrame);
                    Texture_ChoseTint(targetFrame, itemLocation, currentItemID, itemID);
                end
            end
        else
            self:Overlay_Hide();
        end
    end

    function OverlayManager:UpdateBank()
        bankContents = {};
        
        Bank_ContainerUpdate(BANK_CONTAINER);

        for bagID = NUM_BAG_SLOTS + 1, NUM_BANKBAGSLOTS do
            Bank_ContainerUpdate(bagID);
        end

        self:Overlay_Show();
    end

    function OverlayManager:OnEnable()
        self:SecureHook(nil, "GearSetButton_OnClick", "Overlay_Show");
        self:HookScript(PaperDollEquipmentManagerPane, "OnShow", "Overlay_Show");
        self:HookScript(PaperDollEquipmentManagerPane, "OnHide", "Overlay_Hide");

        self:RegisterEvent("EQUIPMENT_SETS_CHANGED", "Overlay_Show");
        self:RegisterEvent("PLAYER_EQUIPMENT_CHANGED", "Overlay_Show");
        self:RegisterEvent("BANKFRAME_OPENED", "UpdateBank");
        self:RegisterEvent("BANKFRAME_CLOSED", "UpdateBank");
        self:RegisterEvent("PLAYERBANKSLOTS_CHANGED", "UpdateBank");
    end

    function OverlayManager:OnDisable()
        self:UnregisterEvent("PLAYER_EQUIPMENT_CHANGED");
        self:UnregisterEvent("EQUIPMENT_SETS_CHANGED");
        self:UnregisterEvent("BANKFRAME_OPENED");
        self:UnregisterEvent("BANKFRAME_CLOSED");
        self:UnregisterEvent("PLAYERBANKSLOTS_CHANGED");

        self:Unhook(PaperDollEquipmentManagerPane, "OnShow");
        self:Unhook(PaperDollEquipmentManagerPane, "OnHide");
        self:Unhook(nil, "GearSetButton_OnClick");
        
        self:Overlay_Hide();
    end
end