local addonName, addonTable = ...; -- Pulls back the Addon-Local Variables and store them locally.
local LunaEclipse_SpecManager = LibStub("AceAddon-3.0"):NewAddon(addonName, "AceConsole-3.0", "AceEvent-3.0");

addonTable.addonName = addonName;
_G[addonName] = LunaEclipse_SpecManager;

local FORCE_DEBUG = false;

function LunaEclipse_SpecManager:isCompatableWOWVersion()
    if addonTable.WOW_VERSION_MIN and addonTable.WOW_VERSION_MAX then
        local versionRunning = select(4, GetBuildInfo());
		
        return (versionRunning >= addonTable.WOW_VERSION_MIN) and (versionRunning <= addonTable.WOW_VERSION_MAX);
    end
	
    return false;
end

if LunaEclipse_SpecManager:isCompatableWOWVersion() then
    local ADDON_NAME = addonName;
    local ADDON_TITLE = GetAddOnMetadata(ADDON_NAME, "title");
    local ADDON_AUTHOR = GetAddOnMetadata(ADDON_NAME, "author");
    local ADDON_VERSION = GetAddOnMetadata(ADDON_NAME, "version");
    local majorVersion, minorVersion, buildVersion = string.split(".", ADDON_VERSION);

    local currentSpecGroupID = nil;

    local AddonSettings_Defaults = {
        profile = {
            AddonSettings = {
				allowZeroMacros = false,
				enableEquipManager = true,
				enableMacroManager = true,
				useOverlay = true,
            },
        },
    };

    local LDBProfile_Defaults = {
        profile = {
            LDBIconStorage = {
                hide = false,
            },
        },
    };

    addonTable.currentSpecInfo = nil;
    addonTable.lootSpecInfo = nil;
    addonTable.previousSpecInfo = nil;
    addonTable.versionInfo = {
        major = majorVersion,
        minor = minorVersion,
        build = buildVersion,
    };

	local function compareTables(compareTable, comparisonTable)
		local returnValue = true;

	    for key, value in pairs(compareTable) do
            if type(value) == "table" then
				returnValue = LunaEclipse_SpecManager:EquateTables(value, comparisonTable[key]);
            else
				if comparisonTable[key] then
					returnValue = (value == comparisonTable[key]);
				else
					returnValue = false;
				end
            end
				
			if returnValue == false then
				break;
			end
        end

		return returnValue;
	end

    function LunaEclipse_SpecManager:CMDLine(...)
        local profile = addonTable.LESM_LDBProfile.profile;

		profile.LDBIconStorage.hide = not profile.LDBIconStorage.hide;

		self:SendMessage(addonTable.LESM_MINIMAP_TOGGLE);
    end

    function LunaEclipse_SpecManager:AddonInfo(stripTags)
        local returnValue = {
            title = (stripTags and ADDON_TITLE) or string.join("", "|cff1784d1", ADDON_TITLE, addonTable.TEXT_COLOR_TAG_END),
            author = (stripTags and ADDON_AUTHOR) or string.join("", "|cffffd700", ADDON_AUTHOR, addonTable.TEXT_COLOR_TAG_END),
            version = (stripTags and ADDON_VERSION) or string.join("", "|cff1784d1", ADDON_VERSION, addonTable.TEXT_COLOR_TAG_END),
        };

        return returnValue;
    end
	
	function LunaEclipse_SpecManager:CountTable(tableName)
		local returnValue = 0;
		
		if tableName then
			for tableKey, tableValue in pairs(tableName) do
				returnValue = returnValue + 1;
			end
		end
		
        return returnValue;
	end

    function LunaEclipse_SpecManager:DuplicateTables(originalTable)
        local returnValue = {};

        if originalTable then
            for key, value in pairs(originalTable) do
                if type(value) == "table" then
                    returnValue[key] = self:DuplicateTables(value);
                else
                    returnValue[key] = value;
                end
            end
        end
		
        return returnValue;
    end

    function LunaEclipse_SpecManager:EmptyTable(testTable)
        return self:CountTable(testTable) == 0;
    end

    function LunaEclipse_SpecManager:EquateTables(firstTable, secondTable)
        local returnValue = false;

        if firstTable and secondTable then
			-- Compare the contents of the first table to the second table for missing or unequal values
			returnValue = compareTables(firstTable, secondTable);
			
			-- Check to see if the contents were the same
			if returnValue then
				-- Compare the contents of the second table to the first table for missing values
				returnValue = compareTables(secondTable, firstTable);
			end
		end

        return returnValue;
    end

    function LunaEclipse_SpecManager:GetLootSpecInfo(specInfoID)
        local returnValue = nil;		
        local lootSpecialization = GetLootSpecialization();
        local specIndex = specInfoID or (((lootSpecialization == 0 and addonTable.currentSpecInfo) and addonTable.currentSpecInfo.id) or (lootSpecialization ~= 0 and lootSpecialization));
		
        if specIndex then 
            local specID, specName, specDescription, specIcon, specRole, specClass = GetSpecializationInfoByID(specIndex);

            returnValue = {
                id = specID,
                name = specName,
                description = specDescription,
                icon = specIcon,
                role = specRole,
                class = specClass,
            };
        end
		
        return returnValue;
    end

    function LunaEclipse_SpecManager:GetSpecInfo(specInfoID)
        local returnValue = nil;		
        local specIndex = (specInfoID and GetSpecialization(false, false, specInfoID)) or GetSpecialization();

        if specIndex then 
            local specID, specName, specDescription, specIcon, specRole, specPrimaryStat = GetSpecializationInfo(specIndex);

            returnValue = {
				index = specIndex,
                id = specID,
                name = specName,
                description = specDescription,
                icon = specIcon,
                role = specRole,
                primaryStat = specPrimaryStat,
            };
        end
		
        return returnValue;
    end

	function LunaEclipse_SpecManager:InCombat()
		return UnitAffectingCombat("player");
	end

    function LunaEclipse_SpecManager:MergeTables(returnValue, mergeTable)
        if mergeTable then
            for key, value in pairs(mergeTable) do
                returnValue[key] = value;
            end
        end
		
        return returnValue;
    end

    function LunaEclipse_SpecManager:PLAYER_LOGIN(event, ...)
        addonTable.currentSpecInfo = self:GetSpecInfo();
        addonTable.lootSpecInfo = self:GetLootSpecInfo();

 		currentSpecGroupID = GetSpecialization();

        self:SendMessage(addonTable.LESM_LOAD, addonTable.currentSpecInfo);
    end

    function LunaEclipse_SpecManager:UNIT_SPELLCAST_START(event, ...)
        local eventUnit, _, _, _, spellID = select(1, ...);
		
		if eventUnit == "player" and (spellID == addonTable.ACTIVATE_SPEC or spellID == addonTable.ACTIVATE_PRIMARY_SPEC or spellID == addonTable.ACTIVATE_SECONDARY_SPEC) then
            self:SendMessage(addonTable.LESM_SPEC_CHANGE_START, addonTable.currentSpecInfo);
        end
    end

    function LunaEclipse_SpecManager:PLAYER_SPECIALIZATION_CHANGED(event, ...)
		local eventUnit = select(1, ...);

        if eventUnit == "player" then
            local currentSpecInfo = self:DuplicateTables(addonTable.currentSpecInfo);
            local previousSpecGroupID = currentSpecGroupID;
			
            currentSpecGroupID = GetSpecialization();
			
            addonTable.currentSpecInfo = self:GetSpecInfo();
            addonTable.lootSpecInfo = self:GetLootSpecInfo();

            if currentSpecGroupID == previousSpecGroupID then
                if addonTable.currentSpecInfo and not currentSpecInfo then
                    addonTable.previousSpecInfo = nil;

                    self:SendMessage(addonTable.LESM_SPEC_ADDED, addonTable.currentSpecInfo);
                elseif currentSpecInfo and not addonTable.currentSpecInfo then
                    addonTable.previousSpecInfo = self:DuplicateTables(currentSpecInfo);

                    self:SendMessage(addonTable.LESM_SPEC_REMOVED, addonTable.previousSpecInfo);
                end
            else
                if addonTable.currentSpecInfo ~= currentSpecInfo then
                    addonTable.previousSpecInfo = self:DuplicateTables(currentSpecInfo);

                    self:SendMessage(addonTable.LESM_SPEC_CHANGE_FINISH, addonTable.currentSpecInfo);
                end
            end
        end
    end
	
    function LunaEclipse_SpecManager:PLAYER_LOOT_SPEC_UPDATED(event, ...)
        addonTable.lootSpecInfo = self:GetLootSpecInfo();

        self:SendMessage(addonTable.LESM_LOOTSPEC_CHANGED, addonTable.lootSpecInfo);
    end

    function LunaEclipse_SpecManager:PLAYER_REGEN_DISABLED(event, ...)
        self:SendMessage(addonTable.LESM_ENTER_COMBAT, addonTable.currentSpecInfo);
    end

    function LunaEclipse_SpecManager:PLAYER_REGEN_ENABLED(event, ...)
        self:SendMessage(addonTable.LESM_LEFT_COMBAT, addonTable.currentSpecInfo);
    end

    function LunaEclipse_SpecManager:PLAYER_LOGOUT(event, ...)
        self:SendMessage(addonTable.LESM_CLOSE, addonTable.currentSpecInfo);
    end

    function LunaEclipse_SpecManager:OnInitialize()		
        local addonInfo = self:AddonInfo();

		addonTable.addonSettings = LibStub("AceDB-3.0"):New("LESM_AddonSettings", AddonSettings_Defaults, "Settings");
        addonTable.LDBProfile = LibStub("AceDB-3.0"):New("LESM_LDBProfile", LDBProfile_Defaults, "Settings");
        addonTable.macroProfile = LibStub("AceDB-3.0"):New("LESM_MacroProfile", nil, "Settings");
        addonTable.savedProfiles = LibStub("AceDB-3.0"):New("LESM_SavedProfiles", nil, "Settings");

        print(string.format("%s %s %s %s", addonInfo.title, "version:", addonInfo.version, "loaded!"));
    end

    function LunaEclipse_SpecManager:OnEnable()
        self:RegisterEvent("PLAYER_LOGIN");
        self:RegisterEvent("UNIT_SPELLCAST_START");
        self:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED");
        self:RegisterEvent("PLAYER_LOOT_SPEC_UPDATED");
        self:RegisterEvent("PLAYER_REGEN_DISABLED");
        self:RegisterEvent("PLAYER_REGEN_ENABLED");
        self:RegisterEvent("PLAYER_LOGOUT");
		
        self:RegisterChatCommand("LESM", "CMDLine");
    end

    function LunaEclipse_SpecManager:OnDisable()
        self:UnregisterChatCommand("LESM");

        self:UnregisterEvent("PLAYER_LOGIN");
        self:UnregisterEvent("UNIT_SPELLCAST_START");
        self:UnregisterEvent("PLAYER_SPECIALIZATION_CHANGED");
        self:UnregisterEvent("PLAYER_LOOT_SPEC_UPDATED");
        self:UnregisterEvent("PLAYER_REGEN_DISABLED");
        self:UnregisterEvent("PLAYER_REGEN_ENABLED");
        self:UnregisterEvent("PLAYER_LOGOUT");
    end
end